/*
 * inverter.c
 *
 *  Created on: 19.09.2016
 *      Author: dehottgw
 */


#include <stdint.h>
#include <stdlib.h>
#include <msp430g2553.h>
#include <isr_compat.h>

#include "inverter.h"

#define NUM_OF_SINE_VALUES 10

float sineValues[NUM_OF_SINE_VALUES] = {
		0.3090,
		0.5878,
		0.8090,
		0.9511,
		1.0000,
		0.9511,
		0.8090,
		0.5878,
		0.3090,
		0.0000,
};

volatile uint16_t pulseWidths[NUM_OF_SINE_VALUES * 2] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
volatile uint8_t pulseWidthIdx = 0;
volatile uint8_t activeOffset = 0;



ISR(TIMER1_A0, TA1_ISR_Ovrfl) {
	if (pulseWidthIdx == 0) {
		P1OUT ^= BIT1;
	}

	uint16_t pw = pulseWidths[pulseWidthIdx + activeOffset];
	if (pw != 0) {
		TA1CCR1 = pw;
		P2OUT |= BIT1;
	}

	pulseWidthIdx++;
	if (pulseWidthIdx >= NUM_OF_SINE_VALUES) {
		pulseWidthIdx = 0;
	}
}

ISR(TIMER1_A1, TA1_ISR_Comp) {
	if (TA1IV & TA1IV_TACCR1) {
		P2OUT &= ~BIT1;
	}
}



void inverterInit() {
	P1OUT &= ~BIT1;
	P1DIR |= BIT1;

	P2OUT &= ~BIT1;
	P2DIR |= BIT1;

	P2OUT &= ~BIT0;
	P2DIR |= BIT0;
	P2SEL |= BIT0;

	TA1CTL = MC_1 | ID_1 | TASSEL_2 | TACLR;
	TA1CCTL0 = CCIE | OUTMOD_4;
	TA1CCTL1 = CCIE;
	TA1CCR0 = 0;
	TA1CCR1 = 0;
}

void inverterSetFrequency(uint16_t f) {
	float ff = (float)f;
	float slotLength = 100.0 / (ff * 9.8e-6 * 20);

	uint8_t currentOffset = (activeOffset == 0) ? NUM_OF_SINE_VALUES : 0;
	for (uint8_t i = 0; i < NUM_OF_SINE_VALUES; i++) {
		pulseWidths[i + currentOffset] = (uint16_t)(slotLength * sineValues[i]);
	}

	__disable_interrupt();
	activeOffset = currentOffset;
	TA1CCR0 = (uint16_t)slotLength;
	pulseWidthIdx = 0;
	__enable_interrupt();
}
